<?php
/**
 * DLE Shikimori
 *
 * @copyright 2018 PunPun (https://punpun.name)
 * @version   1.0.0
 * @link      https://punpun.name
 */

define('FOLDER_PREFIX', date('Y-m') . '/');

class Shikimori
{
	private static $instance = null;
	private $stats = [];
	
	private $dleConfig, $shikiConfig, $shikiLang, $db, $member_id;
	
	/**
    * Singleton
    *
    * @return object
    */
    static function realize()
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }
		
        return self::$instance;
    }
	
	/**
    * Конфигурация
    *
    * @return object
    */
    function configuration($dleConfig, $shikiConfig, $shikiLang, $db, $member_id, $news_id = 0)
    {
        $this->dleConfig = $dleConfig;
		$this->shikiConfig = $shikiConfig;
		$this->shikiLang = $shikiLang;
		$this->db = $db;
		$this->member_id = $member_id;
		$this->news_id = $news_id;

        return self::$instance;
    }
	
	
	/**
    * curl
    *
	* @param  string  $url
    * @return mixed
    */
	function curl($url)
	{
		$ch = curl_init(); 
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_FAILONERROR, 1);
		curl_setopt($ch, CURLOPT_USERAGENT, $this->dleConfig['http_home_url']);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 4);
		$result = curl_exec($ch);
		curl_close($ch);   
		return $result; 
	}
	
	/**
    * Функция склонения слов
    *
	* @param  int     $count
	* @param  array   $textArray
    * @return string
    */
	function declination($count, $textArray)
	{
		$case = [2, 0, 1, 1, 1, 2];
		return ($count != 0) ? $count . ' ' . ($textArray[($count % 100 > 4 && $count % 100 < 20) ? 2 : $case[min($count % 10, 5)]]) : false;
	}
	
	/**
    * Функция поиска аниме
    *
	* @param  string  $name
    * @return array
    */
	function searchAnime($name)
	{
		$search = ['search' => $name, 'censored' => 'false', 'limit' => 6, 'order' => 'name'];
		$search = 'https://shikimori.org/api/animes?' . http_build_query($search, '', '&');
		$getAnime = $this->curl($search);
		$getAnime = json_decode($getAnime, true);
		
		if (count($getAnime)) {
			foreach ($getAnime as $key => $item) {
				$year = date('Y', strtotime($item['aired_on']));
				$name = $item['russian'] != '' ? $item['russian'] : $item['name'];
				$url = 'https://shikimori.org' . $item['url'];
				$kind = $this->shikiLang['kind'][$item['kind']];
				$episodes = $item['episodes'] == 0 ? $this->shikiLang['dont_know'] : $this->declination($item['episodes'], [$this->shikiLang['episod_1'], $this->shikiLang['episod_2'], $this->shikiLang['episod_5']]);
$arrayAnime[] = <<<HTML
<div style="padding:2px;">
	<a href="#" onclick="parseAnime('{$item['id']}'); return false;">{$name}</a> [{$episodes} , {$year}, {$kind}] { <i><a href="{$url}" target="_blank">Ссылка</a></i> }
</div>
HTML;
			}
			$arrayAnime = implode($arrayAnime);
			return ['anime' => $arrayAnime];
		} else {
			return ['error' => $this->shikiLang['not_found']];
		}
	}
	
	/**
    * Функция парсинга аниме
    *
	* @param  mixed  $id
    * @return array
    */
	function parseAnime($id)
	{
		$getAnime = $this->curl('https://shikimori.org/api/animes/' . $id);
		$getAnime = json_decode($getAnime, true);
		if (count($getAnime)) {
			$arrayAnime = [];
			foreach ($this->shikiConfig['xf'] as $key => $value) {
				if ($key == 'originalName' && $getAnime['name']) {
					$arrayAnime['xf_' . $value] = $getAnime['name'];
				}
				if ($key == 'rusName' && $getAnime['russian']) {
					$arrayAnime['xf_' . $value] = $getAnime['russian'];
				}
				if ($key == 'year' && $getAnime['aired_on']) {
					$arrayAnime['xf_' . $value] = date('Y', strtotime($getAnime['aired_on']));
				}
				if ($key == 'type' && $getAnime['kind']) {
					$arrayAnime['xf_' . $value] = $this->shikiLang['kind'][$getAnime['kind']];
				}
				if ($key == 'status' && $getAnime['status']) {
					$arrayAnime['xf_' . $value] = $this->shikiLang['status'][$getAnime['status']];
				}
				if ($key == 'seriaNum' && $getAnime['episodes']) {
					$arrayAnime['xf_' . $value] = $getAnime['episodes'] == 0 ? $this->shikiLang['dont_know'] : $this->declination($getAnime['episodes'], [$this->shikiLang['episod_1'], $this->shikiLang['episod_2'], $this->shikiLang['episod_5']]);
				}
				if ($key == 'age' && $getAnime['rating']) {
					$arrayAnime['xf_' . $value] = $this->shikiLang['age'][$getAnime['rating']];
				}
				if ($key == 'time' && $getAnime['duration']) {
					$arrayAnime['xf_' . $value] = $getAnime['duration'];
				}
				if ($key == 'rating' && $getAnime['score']) {
					$arrayAnime['xf_' . $value] = $getAnime['score'];
				}
				if ($key == 'description' && $getAnime['description']) {
					$arrayAnime['xf_' . $value] = $getAnime['description'];
				}
				if ($key == 'genres' && $getAnime['genres'][0]) {
					$genres = [];
					foreach ($getAnime['genres'] as $genreIndex => $genreItem) {
						$genres[] = $genreItem['russian'];
					}
					$arrayAnime['xf_' . $value] = implode(', ', $genres);
				}
				if ($key == 'studio' && $getAnime['studios'][0]) {
					$studios = [];
					foreach ($getAnime['studios'] as $studiosIndex => $studiosItem) {
						$studios[] = $studiosItem['filtered_name'] ? $studiosItem['filtered_name'] : $studiosItem['name'];
					}
					$arrayAnime['xf_' . $value] = implode(', ', $studios);
				}
				if ($key == 'trailer' && $getAnime['videos'][0]) {
					$arrayAnime['xf_' . $value] = $getAnime['videos'][0]['player_url'] ? $getAnime['videos'][0]['player_url'] : $getAnime['videos'][0]['url'];
				}
				if ($key == 'idShiki' && $getAnime['id']) {
					$arrayAnime['xf_' . $value] = $getAnime['id'];
				}
				if ($key == 'allOtherName') {
					$allName = [];
					if ($getAnime['english'][0]) {
						$allName[] = implode(', ', $getAnime['english']);
					}
					if ($getAnime['japanese'][0]) {
						$allName[] = implode(', ', $getAnime['japanese']);
					}
					if ($getAnime['synonyms'][0]) {
						$allName[] = implode(', ', $getAnime['synonyms']);
					}
					if ($allName) {
						$arrayAnime['xf_' . $value] = implode(', ', $allName);
					}
				}
			}
			$cats = [];
			foreach ($getAnime['genres'] as $genreIndex => $genreItem) {
				if ($this->shikiConfig['category'][$genreItem['id']]) {
					$cats[] = $this->shikiConfig['category'][$genreItem['id']];
				}
			}
			$arrayAnime['cats'] = $cats;
			
			if ($this->shikiConfig['options']['uploadPoster'] && $this->shikiConfig['options']['xfPoster'] && $getAnime['image']['original']) {
				$arrayAnime['xf_' . $this->shikiConfig['options']['xfPoster']] = $this->parseImage($getAnime['image']['original']);
			}
			if ($this->shikiConfig['options']['uploadScreens'] && $this->shikiConfig['options']['xfScreens'] && $this->shikiConfig['options']['countScreens'] > 0 && $getAnime['screenshots'][0]['original'] != '') {
				$arrayAnime['xf_' . $this->shikiConfig['options']['xfScreens']] = $this->parseScreens($getAnime['id']);
			}
			return $arrayAnime;
		} else {
			return ['error' => $this->shikiLang['not_found']];
		}
	}
	
	/**
    * Функция парсинга изображений
    *
	* @param  string  $img
	* @param  bool    $screens
    * @return string
    */
	function parseImage($img, $screens = false)
	{
		$type = end(explode('.', $img));
		$type = explode('?', $type);
		if (count($type) == 2 && in_array($type[0], ['png', 'jpg', 'jpeg'])) {
			$urlPoster ='https://shikimori.org' . $img;
			
			if (!is_dir(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX)) {
				mkdir(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX, 0777);
				chmod(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX, 0777);
				mkdir(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . 'thumbs', 0777);
				chmod(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . 'thumbs', 0777);
			}
			if (stripos($img, '.php') !== false) {
				return false;
			}
			if (stripos($img, '.phtm') !== false) {
				return false;
			}
			
			$filename = end(explode('/', $img));
			$filename = explode('?', $filename);
			if (count($filename) == 2) {
				$filename = $filename[0];
				if (is_dir(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX)) {
					$file_prefix = time() + rand(1, 100);
					$file_prefix .= '_';
					$filename = $file_prefix . $filename;
					if (copy($urlPoster, ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename)) {
						chmod(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename, 0666);
						
						$added_time = time();
						$sizeInfo = getimagesize(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename);
						if (!in_array($sizeInfo[2], [2, 3])) {
							unlink(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename);
							return false;
						}
						
						$thumb = new thumbnail(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename);
						$this->member_id['name'] = $this->db->safesql($this->member_id['name']);
						$row = $this->db->super_query("SELECT id, images FROM " . PREFIX . "_images WHERE news_id='{$this->news_id}' AND author='{$this->member_id['name']}'");
						if (!$row['id']) {
							$inserts = FOLDER_PREFIX . $filename;
							$this->db->query("INSERT INTO " . PREFIX . "_images (images, author, news_id, date) values ('{$inserts}', '{$this->member_id['name']}', '{$this->news_id}', '{$added_time}')" );
						} else {
							$error_image = false;
							if ($row['images']) {
								$listimages = explode('|||', $row['images']);
								foreach ($listimages as $dataimages) {
									if ($dataimages == FOLDER_PREFIX . $filename) {
										$error_image = true;
										break;
									}
								}
							}
							
							if (!$error_image) {
								if (!$listimages) {
									$listimages = [];
								}
								$listimages[] = FOLDER_PREFIX . $filename;
								$row['images'] = implode('|||', $listimages);
								
								if (dle_strlen($row['images'], $this->dleConfig['charset']) < 65000) {
									$this->db->query("UPDATE " . PREFIX . "_images SET images='{$row['images']}' WHERE news_id='{$this->news_id}' AND author='{$this->member_id['name']}'");
								}
							}
						}
						
						if ($screens) {
							$sizeTumb = $this->shikiConfig['options']['sizeTumbScreens'];
							$typeSizeTumb = $this->shikiConfig['options']['typeSizeTumbScreens'];
							$quality = $this->shikiConfig['options']['qualityScreens'];
							$sizeImage = $this->shikiConfig['options']['sizeScreens'];
						} else {
							$sizeTumb = $this->shikiConfig['options']['sizeTumb'];
							$typeSizeTumb = $this->shikiConfig['options']['typeSizeTumb'];
							$quality = $this->shikiConfig['options']['qualityPoster'];
							$sizeImage = $this->shikiConfig['options']['sizePoster'];
						}
						
						$thumb_bool = false;
						if ($sizeTumb > 0) {
							$tumbSize = explode('x', $sizeTumb);
							if (count($tumbSize) == 2 && $tumbSize[0] && $tumbSize[1]) {
								$tumbSize = intval($tumbSize[0]) . 'x' . intval($tumbSize[1]);
							} else {
								$tumbSize = intval($tumbSize[0]);
							}
							
							if ($thumb->size_auto($tumbSize, $typeSizeTumb)) {
								$thumb->jpeg_quality($quality);
								$thumb->save(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . 'thumbs/' . $filename);
								chmod(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . 'thumbs/' . $filename, 0666);
								$thumb_bool = true;
							}
						}
						
						if ($sizeImage > 0) {
							$thumb = new thumbnail(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename);
							$thumb->jpeg_quality($quality);
							$re_save = $thumb->re_save;
							
							$sizeImage = explode('x', $sizeImage);
							if (count($sizeImage) == 2 && $sizeImage[0] && $sizeImage[1]) {
								$sizeImage = intval($sizeImage[0]) . 'x' . intval($sizeImage[1]);
							} else {
								$sizeImage = intval($sizeImage[0]);
							}
							
							if ($thumb->size_auto($sizeImage, 0)) {
								$re_save = true;
							}
							
							if ($re_save) {
								$thumb->save(ROOT_DIR . '/uploads/posts/' . FOLDER_PREFIX . $filename);
							}
						}
						
						if ($thumb_bool) {
							$m = '[thumb]' . $this->dleConfig['http_home_url'] . 'uploads/posts/' . FOLDER_PREFIX . $filename . '[/thumb]';
						} else {
							$m = $this->dleConfig['http_home_url'] . 'uploads/posts/' . FOLDER_PREFIX . $filename;
						}
						
						return $m;
					}
				}
			}
		}
	}
	
	/**
    * Функция парсинга скриншотов
    *
	* @param  int     $id
    * @return string
    */
	function parseScreens($id)
	{
		$getScreen = $this->curl('https://shikimori.org/api/animes/' . $id . '/screenshots');
		$getScreen = json_decode($getScreen, true);
		
		if (count($getScreen)) {
			$i = 1;
			$screens = [];
			foreach ($getScreen as $screen) {
				$screens[] = $this->parseImage($screen['original'], true);
				if ($i == $this->shikiConfig['options']['countScreens']) {
					break;
				}
				$i++;
			}
			return implode($screens);
		}
		
		return false;
	}
	
	private function __construct() {}
	private function __wakeup() {}
	private function __clone() {}
	private function __sleep() {}
}

$dleShikimori = Shikimori::realize()->configuration($config, $dle_shikimori_config, $shikiLang, $db, $member_id, $newsId);